==> libro-codemirror-0.1.0.tgz: package/es/indentation-markers/index.js <==
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }
function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }
function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return _typeof(key) === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (_typeof(input) !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (_typeof(res) !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
import { getIndentUnit } from '@codemirror/language';
import { RangeSetBuilder } from '@codemirror/state';
import { Decoration, ViewPlugin, EditorView } from '@codemirror/view';
import { indentationMarkerConfig } from "./config.js";
import { IndentationMap } from "./map.js";
import { getCurrentLine, getVisibleLines } from "./utils.js";

// CSS classes:
// - .cm-indent-markers

// CSS variables:
// - --indent-marker-bg-part
// - --indent-marker-active-bg-part

/** Color of inactive indent markers. Based on RUI's var(--background-higher) */
var MARKER_COLOR_LIGHT = '#5f6064';
var MARKER_COLOR_DARK = '#5f6064';

/** Color of active indent markers. Based on RUI's var(--background-highest) */
var MARKER_COLOR_ACTIVE_LIGHT = '#A4AECB';
var MARKER_COLOR_ACTIVE_DARK = '#565C6D';

/** Thickness of indent markers. Probably should be integer pixel values. */
var MARKER_THICKNESS = '1px';
var indentTheme = EditorView.baseTheme({
  '&light': {
    '--indent-marker-bg-color': MARKER_COLOR_LIGHT,
    '--indent-marker-active-bg-color': MARKER_COLOR_ACTIVE_LIGHT
  },
  '&dark': {
    '--indent-marker-bg-color': MARKER_COLOR_DARK,
    '--indent-marker-active-bg-color': MARKER_COLOR_ACTIVE_DARK
  },
  '.cm-line': {
    position: 'relative'
  },
  // this pseudo-element is used to draw the indent markers,
  // while still allowing the line to have its own background.
  '.cm-indent-markers::before': {
    content: '""',
    position: 'absolute',
    top: 0,
    left: 0,
    right: 0,
    bottom: 0,
    background: 'var(--indent-markers)',
    pointerEvents: 'none',
    zIndex: '-1'
  }
});
function createGradient(markerCssProperty, indentWidth, startOffset, columns) {
  var gradient = "repeating-linear-gradient(to right, var(".concat(markerCssProperty, ") 0 ").concat(MARKER_THICKNESS, ", transparent ").concat(MARKER_THICKNESS, " ").concat(indentWidth, "ch)");
  // Subtract one pixel from the background width to get rid of artifacts of pixel rounding
  return "".concat(gradient, " ").concat(startOffset * indentWidth, "ch/calc(").concat(indentWidth * columns, "ch - 1px) no-repeat");
}

// libro 没有使用codemirror的dark theme机制
// const indentMarkerBgColor = '--indent-marker-bg-color';
// const indentMarkerActiveBgColor = '--indent-marker-active-bg-color';
var indentMarkerBgColor = '--mana-libro-editor-indent-marker-bg-color';
var indentMarkerActiveBgColor = '--mana-libro-editor-indent-marker-active-bg-color';
function makeBackgroundCSS(entry, indentWidth, hideFirstIndent) {
  var level = entry.level,
    active = entry.active;
  if (hideFirstIndent && level === 0) {
    return [];
  }
  var startAt = hideFirstIndent ? 1 : 0;
  var backgrounds = [];
  if (active !== undefined) {
    var markersBeforeActive = active - startAt - 1;
    if (markersBeforeActive > 0) {
      backgrounds.push(createGradient(indentMarkerBgColor, indentWidth, startAt, markersBeforeActive));
    }
    backgrounds.push(createGradient(indentMarkerActiveBgColor, indentWidth, active - 1, 1));
    if (active !== level) {
      backgrounds.push(createGradient(indentMarkerBgColor, indentWidth, active, level - active));
    }
  } else {
    backgrounds.push(createGradient(indentMarkerBgColor, indentWidth, startAt, level - startAt));
  }
  return backgrounds.join(',');
}
var IndentMarkersClass = /*#__PURE__*/function () {
  function IndentMarkersClass(view) {
    _classCallCheck(this, IndentMarkersClass);
    this.view = view;
    this.unitWidth = getIndentUnit(view.state);
    this.currentLineNumber = getCurrentLine(view.state).number;
    this.generate(view.state);
  }
  _createClass(IndentMarkersClass, [{
    key: "update",
    value: function update(_update) {
      var unitWidth = getIndentUnit(_update.state);
      var unitWidthChanged = unitWidth !== this.unitWidth;
      if (unitWidthChanged) {
        this.unitWidth = unitWidth;
      }
      var lineNumber = getCurrentLine(_update.state).number;
      var lineNumberChanged = lineNumber !== this.currentLineNumber;
      this.currentLineNumber = lineNumber;
      var activeBlockUpdateRequired = _update.state.facet(indentationMarkerConfig).highlightActiveBlock && lineNumberChanged;
      if (_update.docChanged || _update.viewportChanged || unitWidthChanged || activeBlockUpdateRequired) {
        this.generate(_update.state);
      }
    }
  }, {
    key: "generate",
    value: function generate(state) {
      var builder = new RangeSetBuilder();
      var lines = getVisibleLines(this.view, state);
      var _state$facet = state.facet(indentationMarkerConfig),
        hideFirstIndent = _state$facet.hideFirstIndent,
        markerType = _state$facet.markerType;
      var map = new IndentationMap(lines, state, this.unitWidth, markerType);
      var _iterator = _createForOfIteratorHelper(lines),
        _step;
      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var line = _step.value;
          var entry = map.get(line.number);
          if (!(entry !== null && entry !== void 0 && entry.level)) {
            continue;
          }
          var backgrounds = makeBackgroundCSS(entry, this.unitWidth, hideFirstIndent);
          builder.add(line.from, line.from, Decoration.line({
            class: 'cm-indent-markers',
            attributes: {
              style: "--indent-markers: ".concat(backgrounds)
            }
          }));
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }
      this.decorations = builder.finish();
    }
  }]);
  return IndentMarkersClass;
}();
export function indentationMarkers() {
  var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return [indentationMarkerConfig.of(config), indentTheme, ViewPlugin.fromClass(IndentMarkersClass, {
    decorations: function decorations(v) {
      return v.decorations;
    }
  })];
}

==> libro-codemirror-0.2.2.tgz: package/es/indentation-markers/index.js <==
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }
function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }
function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : String(i); }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
import { getIndentUnit } from '@codemirror/language';
import { RangeSetBuilder } from '@codemirror/state';
import { Decoration, ViewPlugin, EditorView } from '@codemirror/view';
import { indentationMarkerConfig } from "./config.js";
import { IndentationMap } from "./map.js";
import { getCurrentLine, getVisibleLines } from "./utils.js";

// CSS classes:
// - .cm-indent-markers

// CSS variables:
// - --indent-marker-bg-part
// - --indent-marker-active-bg-part

/** Color of inactive indent markers. Based on RUI's var(--background-higher) */
var MARKER_COLOR_LIGHT = '#5f6064';
var MARKER_COLOR_DARK = '#5f6064';

/** Color of active indent markers. Based on RUI's var(--background-highest) */
var MARKER_COLOR_ACTIVE_LIGHT = '#A4AECB';
var MARKER_COLOR_ACTIVE_DARK = '#565C6D';

/** Thickness of indent markers. Probably should be integer pixel values. */
var MARKER_THICKNESS = '1px';
var indentTheme = EditorView.baseTheme({
  '&light': {
    '--indent-marker-bg-color': MARKER_COLOR_LIGHT,
    '--indent-marker-active-bg-color': MARKER_COLOR_ACTIVE_LIGHT
  },
  '&dark': {
    '--indent-marker-bg-color': MARKER_COLOR_DARK,
    '--indent-marker-active-bg-color': MARKER_COLOR_ACTIVE_DARK
  },
  '.cm-line': {
    position: 'relative'
  },
  // this pseudo-element is used to draw the indent markers,
  // while still allowing the line to have its own background.
  '.cm-indent-markers::before': {
    content: '""',
    position: 'absolute',
    top: 0,
    left: 0,
    right: 0,
    bottom: 0,
    background: 'var(--indent-markers)',
    pointerEvents: 'none',
    zIndex: '-1'
  }
});
function createGradient(markerCssProperty, indentWidth, startOffset, columns) {
  var gradient = "repeating-linear-gradient(to right, var(".concat(markerCssProperty, ") 0 ").concat(MARKER_THICKNESS, ", transparent ").concat(MARKER_THICKNESS, " ").concat(indentWidth, "ch)");
  // Subtract one pixel from the background width to get rid of artifacts of pixel rounding
  return "".concat(gradient, " ").concat(startOffset * indentWidth, "ch/calc(").concat(indentWidth * columns, "ch - 1px) no-repeat");
}

// libro 没有使用codemirror的dark theme机制
// const indentMarkerBgColor = '--indent-marker-bg-color';
// const indentMarkerActiveBgColor = '--indent-marker-active-bg-color';
var indentMarkerBgColor = '--mana-libro-editor-indent-marker-bg-color';
var indentMarkerActiveBgColor = '--mana-libro-editor-indent-marker-active-bg-color';
function makeBackgroundCSS(entry, indentWidth, hideFirstIndent) {
  var level = entry.level,
    active = entry.active;
  if (hideFirstIndent && level === 0) {
    return [];
  }
  var startAt = hideFirstIndent ? 1 : 0;
  var backgrounds = [];
  if (active !== undefined) {
    var markersBeforeActive = active - startAt - 1;
    if (markersBeforeActive > 0) {
      backgrounds.push(createGradient(indentMarkerBgColor, indentWidth, startAt, markersBeforeActive));
    }
    backgrounds.push(createGradient(indentMarkerActiveBgColor, indentWidth, active - 1, 1));
    if (active !== level) {
      backgrounds.push(createGradient(indentMarkerBgColor, indentWidth, active, level - active));
    }
  } else {
    backgrounds.push(createGradient(indentMarkerBgColor, indentWidth, startAt, level - startAt));
  }
  return backgrounds.join(',');
}
var IndentMarkersClass = /*#__PURE__*/function () {
  function IndentMarkersClass(view) {
    _classCallCheck(this, IndentMarkersClass);
    this.view = view;
    this.unitWidth = getIndentUnit(view.state);
    this.currentLineNumber = getCurrentLine(view.state).number;
    this.generate(view.state);
  }
  _createClass(IndentMarkersClass, [{
    key: "update",
    value: function update(_update) {
      var unitWidth = getIndentUnit(_update.state);
      var unitWidthChanged = unitWidth !== this.unitWidth;
      if (unitWidthChanged) {
        this.unitWidth = unitWidth;
      }
      var lineNumber = getCurrentLine(_update.state).number;
      var lineNumberChanged = lineNumber !== this.currentLineNumber;
      this.currentLineNumber = lineNumber;
      var activeBlockUpdateRequired = _update.state.facet(indentationMarkerConfig).highlightActiveBlock && lineNumberChanged;
      if (_update.docChanged || _update.viewportChanged || unitWidthChanged || activeBlockUpdateRequired) {
        this.generate(_update.state);
      }
    }
  }, {
    key: "generate",
    value: function generate(state) {
      var builder = new RangeSetBuilder();
      var lines = getVisibleLines(this.view, state);
      var _state$facet = state.facet(indentationMarkerConfig),
        hideFirstIndent = _state$facet.hideFirstIndent,
        markerType = _state$facet.markerType;
      var map = new IndentationMap(lines, state, this.unitWidth, markerType);
      var _iterator = _createForOfIteratorHelper(lines),
        _step;
      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var line = _step.value;
          var entry = map.get(line.number);
          if (!(entry !== null && entry !== void 0 && entry.level)) {
            continue;
          }
          var backgrounds = makeBackgroundCSS(entry, this.unitWidth, hideFirstIndent);
          builder.add(line.from, line.from, Decoration.line({
            class: 'cm-indent-markers',
            attributes: {
              style: "--indent-markers: ".concat(backgrounds)
            }
          }));
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }
      this.decorations = builder.finish();
    }
  }]);
  return IndentMarkersClass;
}();
export function indentationMarkers() {
  var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return [indentationMarkerConfig.of(config), indentTheme, ViewPlugin.fromClass(IndentMarkersClass, {
    decorations: function decorations(v) {
      return v.decorations;
    }
  })];
}
